-- This code is designed to be loaded by .preload.
-- Upgrades and/or convert tables in the CMS database.

local args = {...}
local version = args[1]

do
   local fmt=string.format
   local env, conn

   function traceUpgrade(old,new)
      trace(fmt("-------------> CMS database: Upgrading from version %s to %s",old,new))
   end

   -- Updates the config table with the key value pairs set in cfg
   local function updateConfig(cfg)
      local exec={}
      for k,v in pairs(cfg) do
         table.insert(exec,fmt('UPDATE config SET key="%s",value="%s" WHERE key="%s"',k,v:gsub('"','""'),k))
      end
      local ok,resp,err=conn:mexec(table.concat(exec, ";"))
      if not ok then trace(resp, err) end
   end



   ---------------- Version 1.0 -> 1.1 ------------------------------
   local function v1_0to1_1()
      traceUpgrade("1.0","1.1")
      local ok,err,serr=conn:mexec[[ 
            INSERT INTO config (key, value) values("bloggeruser", "");
            INSERT INTO config (key, value) values("bloggerpasswd", "");
      ]]
      if not ok then
         trace(err,serr)
      end
      conn:mexec[[ 
            INSERT INTO config (key, value) values("version", "1.1");
      ]]
   end


   ---------------- Version 1.1 -> 1.2 ------------------------------
   local function v1_1to1_2()
      traceUpgrade("1.1","1.2")
      local t={"ALTER TABLE blog ADD COLUMN url TEXT;"}
      local cur = conn:execute("SELECT rowid,subject FROM blog")
      local rowid,subject=cur:fetch()
      while rowid do
         subject=subject:gsub(' ','-')
         subject=subject:gsub('"','""')
         table.insert(t,fmt('UPDATE blog SET url="%s" WHERE ROWID=%s;',subject,rowid))
         rowid,subject=cur:fetch()
      end
      cur:close()
      local ok,err,serr=conn:mexec(table.concat(t))
      if not ok then
         trace(err,serr)
      end
   end

   ---------------- Version 1.2 -> 1.3 ------------------------------
   local function v1_2to1_3()
      traceUpgrade("1.2","1.3")
      local q='ALTER TABLE links ADD COLUMN lsp INTEGER;UPDATE links SET lsp=0;INSERT INTO config (key, value) values("blog", "Blog");'
      local ok,err,serr=conn:mexec(q)
      if not ok then
         trace(err,serr)
      end
   end
   
   ---------------- Version 1.3 -> 1.4 ------------------------------
   local function v1_3to1_4()
      traceUpgrade("1.3","1.4")
      local ok,err,serr=conn:mexec(
         'INSERT INTO config (key, value) values("msgrsskey", "");')
      if not ok then
         trace(err,serr)
      end
   end

   ---------------- Version 1.4 -> 1.5 ------------------------------
   local function v1_4to1_5()
      traceUpgrade("1.4","1.5")
      local ok,err,serr=conn:mexec(
         'INSERT INTO config (key, value) values("fbkey", "");INSERT INTO config (key, value) values("fblink", "Add a comment");')
      if not ok then
         trace(err,serr)
      end
   end
   local function v1_5to1_6()
      traceUpgrade("1.5","1.6")
      local ok,err,serr=conn:mexec[[
CREATE TABLE PhotoAlbumNames(key INTEGER PRIMARY KEY,name TEXT NOT NULL, UNIQUE (name));
CREATE TABLE PhotoAlbums(key INTEGER PRIMARY KEY, album INTEGER REFERENCES PhotoAlbumNames(key) NOT NULL,
                          img TEXT NOT NULL, info TEXT NOT NULL, UNIQUE (img));]]
      if not ok then
         trace(err,serr)
      end
   end

   local function v1_6to1_7()
      traceUpgrade("1.6","1.7")
      local q='ALTER TABLE links ADD COLUMN parent INTEGER DEFAULT 0;'
      local ok,err,serr=conn:mexec(q)
      if not ok then
         trace(err,serr)
      end
   end

   local function v1_7to1_8()
      traceUpgrade("1.7","1.8")
      local q=[[ALTER TABLE links ADD COLUMN iscat INTEGER DEFAULT 0;
               UPDATE links SET iscat=1 WHERE data IS NULL;
               UPDATE links SET data='' WHERE data IS NULL;]]
      local ok,err,serr=conn:mexec(q)
      if not ok then
         trace(err,serr)
      end
   end

   ------ Run all upgrades
   env, conn = openDb()
   if not config.version or config.version < "1.1" then
      v1_0to1_1()
      config.version="1.1"
   end
   if config.version < "1.2" then
      v1_1to1_2()
      config.version="1.2"
   end
   if config.version < "1.3" then
      v1_2to1_3()
      config.version="1.3"
   end
   if config.version < "1.4" then
       v1_3to1_4()
      config.version="1.4"
   end
   if config.version < "1.5" then
       v1_4to1_5()
      config.version="1.5"
   end
   if config.version < "1.6" then
       v1_5to1_6()
      config.version="1.6"
   end
   if config.version < "1.7" then
       v1_6to1_7()
      config.version="1.7"
   end
   if config.version < "1.8" then
       v1_7to1_8()
      config.version="1.8"
   end
   updateConfig({version=version})
   closeDb(env, conn)
end
