<?lsp

local x=require"xmlrpc"
 

local bio = bd.baseio
local fmt=string.format
local env,conn
local sfind=string.find

local function authenticate(username, password)
   local c = app.config
   local u = c.bloggeruser
   local p = c.bloggerpasswd

   if u and p and #u ~= 0 and #p  ~= 0 and u == username and p == password then
      return true
   end
   return false
end

local function loginErr()
   return nil, 801, "bad username/password"
end

local function title2url(s)
   return s:gsub("[^%a%d%s]", ""):gsub("^%s*(.-)%s*$", "%1"):gsub("%s+", "-")
end

-- Returns path1 + path2
local function addPath(path1, path2)
   local path = fmt("%s/%s",path1,path2)
   path=path:gsub("//","/") -- just in case
   return path
end

local function cmsPath(path)
   return addPath(app.cmsdocsName,path)
end

local function mkCmsDir(path)
   path=cmsPath(path)
   if not bio:stat(path) then
      print(path)
      return bio:mkdir(path)
   end
   return true
end


-- Ms Word fails when fetching its own blogs with abs URL
local function url2uri(path)
   local function f(a,b)
      if sfind(b,"[hH][tT][tT][pP]") ~= 1 then
         local host = request:header"host"
         if host then
            b=fmt("http%s://%s%s",request:issecure() and "s" or "",host,b)
         end
      end
      return fmt("%s'%s'",a,b)
   end
   return path:gsub('(<img.-src=)[\'"](.-)["\']',f)
end


-- split PathName into its components and create each directory.
local function mkCmsPathForName(pn)
   local e
   local npath="";
   
   local pelems=0
   for n in string.gmatch(pn, "[^/]+") do
      pelems = pelems + 1
   end

   for n in string.gmatch(pn, "[^/]+") do
      pelems = pelems -1
      if pelems < 1 then
         break
      end
      npath = addPath(npath,n)
      local na
      na,e = mkCmsDir(npath)
   end
   if e then
      return nil,"Cannot create \""..npath.."\""
   end
   return true
end


-- Removes our own domainname that is created by the external editors. i.e. convert URL to a URI
local function removeDomainName(message)
   local url=fmt("http[s]?://%s",request:domain())
   return string.gsub(message, url, "")
end

local function getCmsUrl()
   return fmt("http%s://%s/",request:issecure() and "s" or "",request:domain())
end


--string metaWeblog.newPost(
--   string blogid,string username,string password,struct content,bool publish)
local function newPost(blogid, username, password, c, publish)
--trace("newPost",blogid, username, password, c, publish)
   if not authenticate(username, password) then return loginErr() end
   local subject=c.title
   local url = title2url(subject)
   local data = c.description
   local date = (type(c.dateCreated) == "number" and c.dateCreated) or
      (type(c.date_created_gmt) == "number" and c.date_created_gmt)
      or os.time()
   local category = c.categories and c.categories[1] or ""
   local na,e

   local sql=fmt("%s values(-1,0,%u,'%s','/%s','%s',0)",
                 "INSERT INTO links (position,parent,date,lname,url,data,lsp)",
                 date,
                 subject:gsub("'","''"),
                 url:gsub("'","''"),
                 removeDomainName(data:gsub("'","''")))
   na,e,x=conn:execute(sql)
   if e then return nil, 802, e end
   app.updateArchList()
   app.emptyPageCache()
   return tostring(conn:getlastautoid())
end



--bool metaWeblog.editPost(string postid,string username,string password,struct content,bool publish)
local function editPost(postid, username, password, c, publish)
--trace("editPost",postid, username, password, c, publish)
   if not authenticate(username, password) then return loginErr() end
   local id = tonumber(postid)
   local subject=c.title
   local data = c.description
   local date = (type(c.dateCreated) == "number" and c.dateCreated) or
      (type(c.date_created_gmt) == "number" and c.date_created_gmt)
      or os.time()
   local cur,e=conn:execute(fmt([[SELECT DISTINCT date FROM links WHERE ROWID=%s]],id))
   e=true
   if cur then
      if cur:fetch() then
         e=false
      end
      cur:close()
   end
   if e then
      return nil, 802, "No Such Blog"
   end
   cur,e=conn:execute(fmt(
      [[UPDATE links SET date="%u", lname='%s', data='%s' WHERE ROWID=%s]],
      date,
      subject:gsub("'","''"), 
      removeDomainName(data:gsub("'","''")),
      id))
   if e then
      return nil, 802, e
   end
   app.updateArchList()
   app.emptyPageCache()
   return true
end


--struct metaWeblog.getPost(string postid,string username,string password)
local function getPost(postid, username, password)
--trace("getPost",ostid, username, password)
   if not authenticate(username, password) then return loginErr() end
   local id = tonumber(postid)

   local cur,e=conn:execute(fmt(
      [[SELECT DISTINCT date,lname,data FROM links WHERE ROWID=%s]],id))
   if e then return nil, 802, e end
   if cur then
      local s={}
      local data
      s.dateCreated,s.title,data = cur:fetch()
      s.description=url2uri(data)
      cur:close()
      if s.dateCreated then
         s.dateCreated = x.iso8601(s.dateCreated)
         s.categories={}
         return s
      end
   end
   return nil, 802, "No Such Blog"
end


--struct[] metaWeblog.getCategories(string blogid,string username,string password)
local function getCategories(blogid, username, password)
   return {}
end


--struct[] metaWeblog.getRecentPosts(string blogid,string username,string password,int numberOfPosts)
local function getRecentPosts(blogid, username, password, numberOfPosts)
--trace("getRecentPosts", username, password, numberOfPosts)
   if not authenticate(username, password) then return loginErr() end
   numberOfPosts = tonumber(numberOfPosts)
   local cur,e=conn:execute(fmt(
    [[SELECT rowid,date,lname,data FROM links ORDER BY date DESC limit %d]],
    numberOfPosts))
   if cur then
      local r={}
      local rowid,date,subject,data = cur:fetch()
      while rowid do
         local b = {
            postid=rowid,
            dateCreated=x.iso8601(tonumber(date)),
            title=subject,
            description=url2uri(data),
            categories={},
            publish=true
         }
         table.insert(r, b)
         rowid,date,subject,data = cur:fetch()
      end
      cur:close()
      return r
   end
   return nil, 806, e
end

--struct metaWeblog.newMediaObject(string blogid,string username,string password,struct)
local function newMediaObject(blogid, username, password, s)
--trace("newMediaObject", blogid, username, password, s)
   if not authenticate(username, password) then return loginErr() end
   if not s.name or not s.bits then
      return nil, 808, "Invalid media type"
   end

   local fn = addPath("blogmedia",string.gsub(s.name, "\\", "/"))

   local ok,e = mkCmsPathForName(fn)
   if ok then
      local cmsfn=cmsPath(fn)
      local fp = bio:open(cmsfn, "wb")
      if fp then
         fp:write(s.bits)
         fp:close()
         return {url=fmt("http://%s/%s",request:domain(),fn)}
      end
      return nil, 808, fmt("Cannot open %s", bio:realpath(cmsfn))
   end
   return nil, 808, e
end

--bool blogger.deletePost(string appkey,string postid,string username,string password,bool publish)
local function deletePost(appkey, postid, username, password, publish)
--trace("deletePost",appkey, postid, username, password, publish)
   if not authenticate(username, password) then return loginErr() end
   local na,e=conn:execute(fmt(
              [[DELETE FROM links WHERE rowid = %s]], tonumber(postid)),1000)
   if e then 
      return nil, 802, "No Such Blog: "..e
   end
   app.emptyPageCache()
   return true
end

--struct[] blogger.getUsersBlogs(string appkey,string username,string password)
local function getUsersBlogs(appkey, username, password)
--trace("getUsersBlogs", appkey, username, password)
   if not authenticate(username, password) then return loginErr() end
   return {{blogid="cms",blogName=fmt("BD CMS %s",request:domain()), url=getCmsUrl()}}
end


--struct[] blogger.getUserInfo(string appkey,string username,string password)
local function getUserInfo(appkey, username, password)
--trace("getUserInfo",appkey, username, password)
   if not authenticate(username, password) then return loginErr() end
   return {
      email="",
      firstname=username,
      lastname="",
      nickname=username,
      userid=username,
      url=getCmsUrl()
   } 
end

local service = x.new("metaWeblog", {
  metaWeblog={
     newPost=newPost,
     editPost=editPost,
     getPost=getPost,
     getCategories=getCategories,
     getRecentPosts=getRecentPosts,
     newMediaObject=newMediaObject
  },
  blogger={
     deletePost=deletePost,
     getUsersBlogs=getUsersBlogs,
     getUserInfo=getUserInfo
  },
})

response:reset()
request:allow({"POST","GET"},true)
if request:method() == "HEAD" then
   response:setcontentlength(0)
   return
end
if request:method() == "POST" then
   env,conn = app.openDb()
   service:execute(request, response)
   app.closeDb(env,conn)
   return
end
local url=fmt("http%s://%s/editcms/rsd.lsp",request:issecure() and "s" or "",request:domain())
?>
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xmlns:fb="http://www.facebook.com/2008/fbml">
<head>
<meta http-equiv="content-type" content="text/html; charset=utf-8" />
<title>RSD</title>
<link rel="EditURI" type="application/rsd+xml" title="RSD" href="<?lsp=url?>"/>
</head>
<body>
<a href="<?lsp=url?>">RSD</a>
</body>
</html>

