<?lsp
local fmt=string.format

-- Spam control is divided into two checks, one that is always enabled
-- and another that can be enabled by the user.

-- Spam control 1:
-- The first spam control sets 4 hidden variables in the HTML
-- form. The variables are two keys and two encrypted keys. The first
-- key is the time and the second key is a random number.
-- The keys and the encrypted keys are loaded dynamically by the
-- browser, just after the form is loaded. JavaScript uses the AJAX
-- loaded keys to populate the form.
-- The purpose with "spam control 1" is to make it virtually
-- impossible to post a message to feedback.lsp without first loading
-- this page -- i.e. make it impossible for an automatic spam script to
-- post a message.

if not page.k then
   page.k=ba.rnd() -- Create random number for protecting the AJAX call.
end
-- Activated by JavaScript code $.getJSON('feedbackform.lsp',{k:k}... below
local k = request:data"k"
if k and tonumber(k) == page.k then -- If AJAX call and key match.
   -- Generate the two keys.
   local d = {
      k1=os.time(),
      k2=ba.rnd()
   }
   -- Generated the two encrypted keys from k1 and k2.
   d.ck1=ba.aesencode(app.spamctrl.aeskey,tostring(d.k1))
   d.ck2=ba.aesencode(app.spamctrl.aeskey,tostring(d.k2))
   response:json(d) -- Return JSON enc. keys
end


-- Give each hidden input used for spam check a unique ID in case we
-- have more than one form. The ID is used by JavaScript code.
page.inpID = page.inpID and (page.inpID+1) or 1

-- Optional spam control 2:
-- Add the hidden encrypted value of x+y and ask how much x+y is if
-- turing test is enabled
function turingTest()
   local x=ba.rnd(10)
   local y=ba.rnd(10)
   if not app.cfg.SpamPrev then
      return
   end
   local q = string.sub(string.gsub(fmt("%d + %d",x,y), "(.)",
                                     function (c)
                                        return fmt("'0x%x',",string.byte(c))
                                     end),
                         1, -2)
   response:write("<tr><td><b><input name='R' type='hidden' value='",
                  ba.aesencode(app.spamctrl.aeskey,tostring(x+y)),
                  "' />",
                  string.gsub(fmt("%d + %d",x,y), "(.)",
                    function (c)
                          return fmt("&#x%04X", string.byte(c))
                    end),
                  " = </b></td><td><input name='A' type='text' size='30' maxlength='64' /> (Required)</td></tr>")
end
?>
<div id="feedbackform" style="display:none;" align="left">
<fieldset>
<legend>Send Message</legend>
<form method="post" action="/feedback.lsp">
<table>
<tr><td><b>Name:</b></td><td><input name="name" type="text" id="name" size="30" maxlength="64" /> (Required)</td></tr>
<?lsp turingTest() ?>
<tr><td><b>Email Address:</b></td><td><input name="email" type="text" id="email" size="30" maxlength="64" /></td></tr>
</table>
<b>Your Message:</b><br/>
<textarea name="message" style="width:100%" rows="15" wrap="physical">
</textarea>
<input name='k1' type='hidden' value='' id='k1<?lsp=page.inpID?>'/>
<input name='k2' type='hidden' value='' id='k2<?lsp=page.inpID?>'/>
<input name='ck1' type='hidden' value='' id='ck1<?lsp=page.inpID?>'/>
<input name='ck2' type='hidden' value='' id='ck2<?lsp=page.inpID?>'/>
<p><input type="Submit" value="Submit"/></p>
</form>
</fieldset>
</div>
<script>
var id='<?lsp=page.inpID?>';
var k='<?lsp=page.k?>';
$.getJSON('/feedbackform.lsp',{k:k},function(d){
   $('#k1'+id).attr("value", d.k1);
   $('#k2'+id).attr("value", d.k2);
   $('#ck1'+id).attr("value", d.ck1);
   $('#ck2'+id).attr("value", d.ck2);
});
</script>
