<?lsp

local fmt=string.format

function mkUPnPErr(type, code)
   local err={
      [718]="ConflictInMappingEntry<br> The port mapping entry specified conflicts with a mapping assigned previously to another client.",
      [724]="SamePortValuesRequired<br> Internal and External port values must be the same.",
      [714]="NoSuchEntryInArray<br> Internet Gateway Device (router) ignored port mapping command."
   }
   local msg = err[code]
   if not msg then
      msg = fmt("UPnP error code %d.", code)
   end
   return fmt("<p><b>%s port mapping error:</b> %s</p>", type, msg)
end

function extIsNotConnected()
   return "<p style='color:orange'><b>Warning:</b> The external WAN side of your router appears to be disconnected - i.e. you are not connected to the Internet. Check your cable/DSL modem..</p>"
end

function extIsPrivNetwork()
   return "<p style='color:orange'><b>Warning:</b> The external WAN IP address is not directly connected to the Internet. We have detected that you have at least two routers in a chain. You must manually configure port forwarding in the router that is connected to the Internet. Carefully read the <a href='PortForward.lsp' target='_blank'>Manual Port Mapping</a> instructions for how to manually configure port mapping in the router connected to the Internet. Note that you cannot use the FuguHub DDNS client when you have two routers in a chain. We strongly suggest having the server connected to the first router.</p>"
end


local function addportmapping(igd, wanPort, lanPort)
   igd:deleteportmapping(wanPort)
   local ok, ecode = igd:addportmapping(wanPort,lanPort)
   if ok then
      return true
   end
   if ecode == 718 then
      if igd:deleteportmapping(wanPort) then
         ok, ecode = igd:addportmapping(wanPort,lanPort)
         if ok then
            return true
         end
      end
   end
   return false, ecode
end

local error
local warning
local wanIP
local lanIP
local data = request:data()
local igdURL=data.igd
local igd=upnpc.open(igdURL)
if igd then
   wanIP=igd:extip()
   lanIP=igd:sockname()
   if not igd:connected() or wanIP == "0.0.0.0" then
      warning=extIsNotConnected()
   elseif app.ipOnPrivNetwork(wanIP) then
      warning=extIsPrivNetwork()
   end
   local wanHttp, wanHttps = bd.ddns().wanPorts(tonumber(data.http), tonumber(data.https))
   local _, httpErr = addportmapping(igd,wanHttp,bd.port)
   local _, httpsErr = addportmapping(igd,wanHttps,bd.sslport)
   if httpErr then
      error=mkUPnPErr("HTTP",httpErr)
   end
   if httpsErr then
      error = error and error or ""
      error=error..mkUPnPErr("HTTPS",httpsErr)
   end
   igd:close()
else
   error = fmt("<p>Error: Internet Gateway Device (router) %s not responding.</p>",igdURL)
end

title="Automatic UPnP Port Mapping"
local style=""
if error then
   title=title..": Failed!"
   style="color:red"
else
   local session=request:session(true)
   -- Variables used in TestPortMapping.lsp
   session.testPortMapping=true
   session.upnpMapping=true
   session.lanIP=lanIP
   session.wanIP=wanIP
   if warning then
      title=title..": Warning!"
      style="color:orange"
      session.warning=warning
   end
   bd.ddns().setConfAttr("igd",igdURL)
end

-- Make ".header" load our Wikipedia help engine
headerdata=[[<script src="../help.js"></script>]]
response:include"../wizard/.header.lsp"
?>
<h1 style='<?lsp=style?>'><?lsp=title?></h1>
<?lsp if error then
   print(error)
else
   local wHttp, wHttps = bd.ddns().wanPorts()
if warning then
   print("<p>Port mapping was successful, but the following warning was detected:</p>")
   print(warning)
else
   print("<p>Port mapping was successful.</p>")
end
?>

<p>Port mapping is set to automatic mode. You can at any time change the port mapping settings using the FuguHub Administrator Panel.</p>

<p style="float:right"><a href="TestPortMapping.lsp" title="Next"><img src="../images/next.png" border="0" /></a></p>

<div style="padding:2em;border:3px ridge;margin-left: auto;margin-right: auto;width: 30em;background:#F4F4F4">
<h3>Port forwarding/mapping</h3>
<table border="1" cellpadding="5" style="width:95%">
<tr><th>Type</th><th>WAN IP</th><th>WAN Port</th><th>Local IP</th><th>Local port</th></tr>
<tr><td>HTTP</td><td><?lsp=wanIP?></td><td><?lsp=wHttp?></td><td><?lsp=lanIP?></td><td><?lsp=bd.port?></td></tr>
<tr><td>HTTPS</td><td><?lsp=wanIP?></td><td><?lsp=wHttps?></td><td><?lsp=lanIP?></td><td><?lsp=bd.sslport?></td></tr>
</table>

</div>
<?lsp end ?>

<?lsp
if error then ?>
<h3><a href="<?lsp=request:uri()?>">Try again</a></h3><h3><a href="PortForward.lsp">Continue with manual port forwarding</a></h3>
<?lsp
end
showAbort=true
response:include"../wizard/.footer.lsp"
?>
